(function () {
    console.log("Trying to load Jahia kibana dashboards from proxy");
    var context = window.top.hasOwnProperty('jahiaGWTParameters') && window.top.jahiaGWTParameters.hasOwnProperty('contextPath')? window.top.jahiaGWTParameters.contextPath : '';
    var proxyPrefix =  context + "/modules/kibana-proxy";

    function mutateUrl(url) {
        if (url.includes(proxyPrefix)) {
            return url;
        } else if (url.startsWith("/")) {
            return proxyPrefix + url;
        } else if (url.startsWith(window.location.origin)) {
            return url.replace(window.location.origin, window.location.origin + proxyPrefix)
        } else {
            return url;
        }
    }

    function mutateElement(element) {
        if (element) {
            if (element.src) {
                element.src = mutateUrl(element.src);
            }
            if (element.href) {
                element.href = mutateUrl(element.href);
            }
        }
    }

    // OVERRIDE 4: Some Kibana JS is loading assets using "insertBefore" this overriding allows to rewrite the URL before inserting
    var f = Element.prototype.insertBefore;
    Element.prototype.insertBefore = function(){
        mutateElement(arguments[0]);
        return f.apply(this, arguments);
    };

    // OVERRIDE 5: Some Kibana JS is loading assets using "appendChild" this overriding allows to rewrite the URL before inserting
    var k = Element.prototype.appendChild;
    Element.prototype.appendChild = function(){
        mutateElement(arguments[0]);
        return k.apply(this, arguments);
    };

    // OVERRIDE 6: Kibana is using native "fetch" to do REST or other calls on kibana API, this overriding allows to rewrite the URL
    // before fetching the request
    var originalFetch = fetch;
    fetch = (input, init) => {
        var url = input;

        // input can be a Request object or a directly an URL.
        if (input instanceof Request) {
            url = mutateUrl(input.url);

            init = init || {};
            Object.keys(Request.prototype).forEach(function (value) {
                init[value] = input[value];
            });
            delete init.url;

            // Special handling for POST request, the POST data is not readable and cannot be copied on the proxy request
            // So we use the blob() API from fetch SPEC to create a promise on body reading:
            // flow is: blob promise read -> extract the body -> return a new Request promise with the url rewrited and the body from original request
            // due to the asynchronous nature of "fetch" the final promise will be correctly resolved.
            return input.blob().then(function (blob) {
                if (input.method.toUpperCase() !== 'HEAD' && input.method.toUpperCase() !== 'GET' && blob.size > 0) {
                    init.body = blob;
                }

                return originalFetch(url, init)
            })
        } else {
            url = mutateUrl(url);
        }

        return originalFetch(url, init);
    }
})();


